# QNodeOS experiment data and analysis scripts

## Data files

`analysis/data` contains zip files for each experiment:
- `dqc_20240304` for the DQC experiments
- `multitasking_tomography_20240415` for the multitasking experiment focusing on quantum tomography
- `tomography_20240702` for the quantum tomography experiment (without multitasking)
- `multitasking_batch_20240416` for the multitasking experiment focusing on scaling the number of applications

Each zip contains data for each individual combination of application parameters, consisting of application results (pickle files), CNPU and QNPU traces (CSV files), log files, and QDevice traces and calibration results (HDF5 files).

## Terminology
LT3 is the server node. LT4 is the client node. "Host" is used for CNPU, "QNodeOS" is used for QNPU.

## Analysis scripts
`analysis/` contains Jupyter notebooks that run the analysis, producing numbers and figures used in the paper.

### Preparation
Use Python 3.10, preferably on Linux (such as Ubuntu).
Make sure `pipenv` is installed using `python -m pip install pipenv` **AND NOT** using `sudo apt install pipenv` (the latter will install a broken version of `pipenv`).
If it does not yet already exist, create a folder called `figures` (sibling to `lib` and `notebooks`); here will the generated plots be stored.

If you want interactive plots, also install (using `pip`) `ipywidgets` and `ipympl`.


For the data, unzip the `.zip` files in the `data` folder. Use "extract here" or equivalent such that
you end up with `data/dqc_20240304/alpha2_theta1/...` (among other things) *and not* `data/dqc_20240304/dqc_20240304/alpha2_theta1/...`.

### Usage

Make sure your working directory is `<this-repo>/analysis/`.

First of all, set up the Python virtual environment and install all the required
packages with Pipenv:
```sh
pipenv sync
```

Then, launch a shell:
```sh
pipenv shell
```

Then, also install `netsquid` with
```
pip install netsquid --extra-index-url=https://pypi.netsquid.org
```
(make sure you have a [NetSquid](https://netsquid.org/) account).

Also install `uncertainties` with
```
pip install uncertainties
```

Then, in the shell, launch Jupyter Lab:
```sh
jupyter lab
```

Run the Notebooks in the `notebooks/` folder to analyze and plot the data.


### Notebooks
The `notebooks/` folder contains notebooks for each of the experiments that were done:
- DQC demonstration (Fig 3 in main text of the paper): `dqc.ipynb` for analyzing DQC experiment results, `dqc_simulation.ipynb` for the DQC simulation (producing the latency filter cutoff). Uses the `dqc_20240304` data.
- Multitasking of DQC + tomography (Fig 4c in main text of paper): `multitasking_tomography.ipynb` for the DQC+tomography multitasking experiment, and `tomography.ipynb` for when tomography was run on its own (no DQC apps, no multitasking). Uses the `multitasking_tomography_20240415` data and the `tomography_20240702` data, respectively.
- Multitasking scaling (Fig 4d in main text of paper): `multitasking_batch.ipynb` for analyyzing the experiment results. Uses the `multitasking_batch_20240416` data.
