from enum import Enum, auto

from qutip.states import ket


class TraceChannel(str, Enum):
    PROCMGR_SUBROUTINE_ADDED_P0 = "PROCMGR_SUBROUTINE_ADDED_P0"
    PROCMGR_SUBROUTINE_ADDED_P1 = "PROCMGR_SUBROUTINE_ADDED_P1"
    PROCMGR_SUBROUTINE_ADDED_P2 = "PROCMGR_SUBROUTINE_ADDED_P2"
    PROCMGR_SUBROUTINE_ADDED_P3 = "PROCMGR_SUBROUTINE_ADDED_P3"
    PROCMGR_SUBROUTINE_DONE_P0 = "PROCMGR_SUBROUTINE_DONE_P0"
    PROCMGR_SUBROUTINE_DONE_P1 = "PROCMGR_SUBROUTINE_DONE_P1"
    PROCMGR_SUBROUTINE_DONE_P2 = "PROCMGR_SUBROUTINE_DONE_P2"
    PROCMGR_SUBROUTINE_DONE_P3 = "PROCMGR_SUBROUTINE_DONE_P3"
    PROCMGR_SUBROUTINE_ABORTED_P0 = "PROCMGR_SUBROUTINE_ABORTED_P0"
    PROCMGR_SUBROUTINE_ABORTED_P1 = "PROCMGR_SUBROUTINE_ABORTED_P1"
    PROCMGR_SUBROUTINE_ABORTED_P2 = "PROCMGR_SUBROUTINE_ABORTED_P2"
    PROCMGR_SUBROUTINE_ABORTED_P3 = "PROCMGR_SUBROUTINE_ABORTED_P3"

    QDEVICE_PRODUCE_INI_CMD = "QDEVICE_PRODUCE_INI_CMD"
    QDEVICE_PRODUCE_SQG_CMD = "QDEVICE_PRODUCE_SQG_CMD"
    QDEVICE_PRODUCE_TQG_CMD = "QDEVICE_PRODUCE_TQG_CMD"
    QDEVICE_PRODUCE_ENT_CMD = "QDEVICE_PRODUCE_ENT_CMD"
    QDEVICE_PRODUCE_ENM_CMD = "QDEVICE_PRODUCE_ENM_CMD"
    QDEVICE_PRODUCE_MSR_CMD = "QDEVICE_PRODUCE_MSR_CMD"
    QDEVICE_PRODUCE_CMD = "QDEVICE_PRODUCE_CMD"
    QDEVICE_PRODUCE_OUTCOME = "QDEVICE_PRODUCE_OUTCOME"
    QDEVICE_CONSUME_CMD = "QDEVICE_CONSUME_CMD"

    QNETWORK_SWAP_PULL = "QNETWORK_SWAP_PULL"
    QNETWORK_SWAP_RET = "QNETWORK_SWAP_RET"
    QNETWORK_ENT_PULL = "QNETWORK_ENT_PULL"
    QNETWORK_ENM_PULL = "QNETWORK_ENM_PULL"

    QNP_BQ_DELIVERED = "QNP_BQ_DELIVERED"

    QNPD_COMPLETE_SENT = "QNPD_COMPLETE_SENT"
    QNPD_EXPIRE_SENT = "QNPD_EXPIRE_SENT"
    QNPD_TRACK_SENT = "QNPD_TRACK_SENT"
    QNPD_FORWARD_SENT = "QNPD_FORWARD_SENT"
    QNPD_PASS_SENT = "QNPD_PASS_SENT"
    QNPD_COMPLETE_RECEIVED = "QNPD_COMPLETE_RECEIVED"
    QNPD_EXPIRE_RECEIVED = "QNPD_EXPIRE_RECEIVED"
    QNPD_TRACK_RECEIVED = "QNPD_TRACK_RECEIVED"
    QNPD_FORWARD_RECEIVED = "QNPD_FORWARD_RECEIVED"
    QNPD_PASS_RECEIVED = "QNPD_PASS_RECEIVED"
    QNPD_TRACK_CACHED = "QNPD_TRACK_CACHED"

    EGP_ENT_OK = "EGP_ENT_OK"
    EGP_ENM_OK = "EGP_ENM_OK"
    EGP_QNP_OK = "EGP_QNP_OK"

    EGPD_CREATE = "EGPD_CREATE"

    PROCESSOR_START_USER_PROCESS = "PROCESSOR_START_USER_PROCESS"
    PROCESSOR_WAIT_USER_PROCESS = "PROCESSOR_WAIT_USER_PROCESS"
    PROCESSOR_FINISH_USER_PROCESS = "PROCESSOR_FINISH_USER_PROCESS"
    PROCESSOR_START_NET_PROCESS = "PROCESSOR_START_NET_PROCESS"
    PROCESSOR_FINISH_NET_PROCESS = "PROCESSOR_FINISH_NET_PROCESS"
    PROCESSOR_CONSUME_OUTCOME = "PROCESSOR_CONSUME_OUTCOME"

    SCHEDULER_HIGH_PRIO_PROCESS_WAITS_ON_LOWER_PRIO = (
        "SCHEDULER_HIGH_PRIO_PROCESS_WAITS_ON_LOWER_PRIO"
    )
    SCHEDULER_ARRIVE_USER_PROCESS = "SCHEDULER_ARRIVE_USER_PROCESS"
    SCHEDULER_ARRIVE_NET_PROCESS = "SCHEDULER_ARRIVE_NET_PROCESS"
    SCHEDULER_SCHEDULE_USER_PROCESS = "SCHEDULER_SCHEDULE_USER_PROCESS"
    SCHEDULER_SCHEDULE_NET_PROCESS = "SCHEDULER_SCHEDULE_NET_PROCESS"


class QDeviceCommand(Enum):
    NOC = -1
    NOP = 0
    INI = 1
    MSR = 4
    ENT = 5
    ENM = 6
    PMG = 7
    # SQGates ("M" prefix stands for "minus")
    X90 = 8 << 4 | 2
    X180 = 16 << 4 | 2
    MX90 = 24 << 4 | 2
    Y90 = 40 << 4 | 2
    Y180 = 48 << 4 | 2
    MY90 = 56 << 4 | 2


class QDeviceOutcome(Enum):
    UNKNOWN_FAILURE = 0
    HARDWARE_FAILURE = 1
    UNSUPPORTED_REQUEST = 2
    INVALID_QUBIT = 3
    ENT_FAIL = 4
    ENT_SYNC_FAIL = 5
    SUCCESS_ZERO = 8
    SUCCESS_PSI_PLUS = 10
    SUCCESS_PSI_MINUS = 11
    SUCCESS_ONE = 12


class SingleQubitState(Enum):
    """Definition of some common states."""

    PLUS_Z = ket("0")
    MINUS_Z = ket("1")
    PLUS_X = (ket("0") + ket("1")).unit()
    MINUS_X = (ket("0") - ket("1")).unit()
    PLUS_Y = (ket("0") + 1j * ket("1")).unit()
    MINUS_Y = (ket("0") - 1j * ket("1")).unit()


class BellState(Enum):
    """Enumeration of all possible Bell states."""

    PhiPlus = auto()
    PhiMinus = auto()
    PsiPlus = auto()
    PsiMinus = auto()
