#!/usr/bin/env bash

ROOT_PATH=$(dirname "$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)")
LIB_PATH=${ROOT_PATH}/lib
SCRIPTS_PATH=${ROOT_PATH}/scripts
NOTEBOOKS_PATH=${ROOT_PATH}/notebooks

# Format libs and scripts using isort and Black
pipenv run isort "${LIB_PATH}"
pipenv run black "${LIB_PATH}"
pipenv run isort "${SCRIPTS_PATH}"
pipenv run black "${SCRIPTS_PATH}"

# Clean output and execution information of Notebooks
# NOTE: nbconvert does not accept directories as arguments, nor does it glob
# '**/*.ipynb' recursively. Moreover, there isn't a way to glob files which is
# very portable across platforms, and the 'shopt -s globstar' only works in
# Bash >= 4. Thus, we use a custom script to loop over files recursively.
pipenv run python "${SCRIPTS_PATH}/nbfmt.py" --clear-output "${NOTEBOOKS_PATH}"

# Format Notebooks using isort and Black
# NOTE: the CLI of isort does not accept notebooks files, thus, we use a custom
# script to sort imports.
pipenv run python "${SCRIPTS_PATH}/nbfmt.py" --sort-imports "${NOTEBOOKS_PATH}"
pipenv run black "${NOTEBOOKS_PATH}"
